
#include "spi.h"
#include "stm32l0xx.h"


#define DMA_CSELR  (*(volatile unsigned int *)(0x400200a8))


volatile uint8_t tx_done;


void spi_init()
{
	/*Enable clock access to GPIOA*/
	RCC->IOPENR |= RCC_IOPENR_IOPAEN;

	/*Set PA0 as output */
	GPIOA->MODER|=(1<<0);
	GPIOA->MODER&=~(1<<1);


	/*Set PA5,PA6,PA7 mode to alternate function*/
	/*PA5*/
	GPIOA->MODER &=~(1U<<10);
	GPIOA->MODER |=(1U<<11);

	/*PA6*/
	GPIOA->MODER &=~(1U<<12);
	GPIOA->MODER |=(1U<<13);

	/*PA7*/
	GPIOA->MODER &=~(1U<<14);
	GPIOA->MODER |=(1U<<15);

	/*Set PA5,PA6,PA7 alternate function type to SPI1 (AF0)*/

	/*PA5*/
	GPIOA->AFR[0] &= ~(1U<<20);
	GPIOA->AFR[0] &= ~(1U<<21);
	GPIOA->AFR[0] &= ~(1U<<22);
	GPIOA->AFR[0] &= ~(1U<<23);

	/*PA6*/
	GPIOA->AFR[0] &= ~(1U<<24);
	GPIOA->AFR[0] &= ~(1U<<25);
	GPIOA->AFR[0] &= ~(1U<<26);
	GPIOA->AFR[0] &= ~(1U<<27);

	/*PA7*/
	GPIOA->AFR[0] &= ~(1U<<28);
	GPIOA->AFR[0] &= ~(1U<<29);
	GPIOA->AFR[0] &= ~(1U<<30);
	GPIOA->AFR[0] &= ~(1U<<31);

	/*Enable clock access to SPI1 module*/
	RCC->APB2ENR |= RCC_APB2ENR_SPI1EN;



	/*Set mode to MASTER*/
	SPI1->CR1 |= (1U<<2);


	/*Select software slave management by
	* setting SSM=1 and SSI=1*/
	SPI1->CR1 |= (1<<8);
	SPI1->CR1 |= (1<<9);

	/*Enable SPI module*/
	SPI1->CR1 |= (1<<6);

}


void spi_transmit(uint8_t *data,uint32_t size)
{
	uint32_t i=0;


	while(i<size)
	{
		 /*Wait for the TXE bit to set in the Status Register*/
		/*This will indicate that the transmit buffer is empty*/
		 while (!((SPI1->SR)&(1<<1))) {};
		   /*Write the data to the Data Register*/
		   SPI1->DR = data[i];
		   i++;
	}
	 /*Wait for the TXE bit to set in the Status Register*/
	 while (!((SPI1->SR)&(1<<1))) {};

	 /*Wait for the BSY bit to reset in the Status Register*/
	 while (((SPI1->SR)&(1<<7))) {};

	 /*Clear OVR flag*/
	(void) SPI1->DR;
	(void) SPI1->SR;
}


void spi_dma_init()
{
	/*Enable clock access to GPIOA*/
	RCC->IOPENR|=RCC_IOPENR_GPIOAEN;


	/*Set PA0 to output and PA5 to PA7 alternate mode*/
	GPIOA->MODER|=GPIO_MODER_MODE0_0|GPIO_MODER_MODE5_1|GPIO_MODER_MODE6_1|GPIO_MODER_MODE7_1;

	GPIOA->MODER&=~(GPIO_MODER_MODE0_1|GPIO_MODER_MODE5_0|GPIO_MODER_MODE6_0|GPIO_MODER_MODE7_0);

	/*Set speed to to very high speed*/

	GPIOA->OSPEEDR|=(GPIO_OSPEEDER_OSPEED5)|(GPIO_OSPEEDER_OSPEED6)|(GPIO_OSPEEDER_OSPEED7);

	/*Set GPIOs (PA5-PA7) to AF0*/

	GPIOA->AFR[0]&=~(GPIO_AFRL_AFSEL5_Msk<<GPIO_AFRL_AFSEL5_Pos|
			GPIO_AFRL_AFSEL6_Msk<<GPIO_AFRL_AFSEL6_Pos|
			GPIO_AFRL_AFSEL7_Msk<<GPIO_AFRL_AFSEL7_Pos);

	/*Enable clock access to SPI1*/
	RCC->APB2ENR|=RCC_APB2ENR_SPI1EN;

	/*Set SPI to following parameters
	 *
	 * Software slave management
	 * SPI to be master mode
	 * Enable DMA_TX
	 * Enable SPI module
	 * */

	/*Set to master, software slave management */
	SPI1->CR1|=SPI_CR1_MSTR|SPI_CR1_SSM|SPI_CR1_SSI;

	/*Enable DMATX*/
	SPI1->CR2|=SPI_CR2_TXDMAEN;

	/*Enable SPI moduel*/
	SPI1->CR1|=SPI_CR1_SPE;

	/*DMA Configuration*/

	/*Enable clock access to DMA1*/
	RCC->AHBENR|=RCC_AHBENR_DMA1EN;

	/*Configure DMA with following parameters
	 * Memory increment mode
	 * Direction: Read from Memory
	 * Transfer complete interrupt
	 * */
	DMA1_Channel3->CCR|=DMA_CCR_MINC|DMA_CCR_DIR|DMA_CCR_TCIE;

	/*Enable the interrupt in NVIC*/
	NVIC_EnableIRQ(DMA1_Channel2_3_IRQn);

	/*Set the peripheral address*/
	DMA1_Channel3->CPAR =(uint32_t)&SPI1->DR;

	/*Set the channel selection*/
	DMA_CSELR|=(0x01<<8);


}


void spi1_tx_dma(uint8_t * data, uint16_t len)
{
	/*Reset Tx flag*/
	tx_done=0;

	/*Set the memory address to be data*/
	DMA1_Channel3->CMAR=(uint32_t)&data[0];

	/*Set the length*/
	DMA1_Channel3->CNDTR=len;

	/*Clear Interrupt flag*/
	DMA1->IFCR =DMA_IFCR_CTCIF3;

	DMA1_Channel3->CCR|=DMA_CCR_EN;

}


uint8_t tx_status()
{
	if(tx_done==0){return TX_Inprogress;}
	else {return Tx_Done;}
}


void cs_low()
	{
	GPIOA->BSRR=GPIO_BSRR_BR_0;
	}

void cs_high()
	{
	GPIOA->BSRR=GPIO_BSRR_BS_0;
	}


void DMA1_Channel2_3_IRQHandler()
{

	if(DMA1->ISR & DMA_ISR_TCIF3)
	{
		/*Set tx_done to 1*/
		tx_done=1;
		/*Clear Interrupt flag*/
		DMA1->IFCR =DMA_IFCR_CTCIF3;

		/*Disable DMA*/
		DMA1_Channel3->CCR&=~DMA_CCR_EN;

	}

}
